.wy-side-nav-search {
    background-color: initial;
}

.wy-nav-side, .wy-nav-top {
    background-color: #24292E;
}

.wy-side-nav-search input[type="text"] {
    border-color: transparent;
}

.wy-nav-content {
    margin: initial;
}

.rst-content div[role=navigation], footer {
   font-size: 0.85em;
   color: #999;
}

.rst-content div[role=navigation] hr {
    margin-top: 6px;
}

footer hr {
    margin-bottom: 6px;
}

.rst-footer-buttons {
    display: none;
}

.versionmodified {
    color: #008000;
}

a.icon-home, a.icon-home:hover {
    display: inline-block;
    padding: 4px 4px 4px 23px;
    background: transparent url(logo-small.svg) center left no-repeat;
    background-size: 1.5em;
    margin-top: 0.2em;
    margin-bottom: 1em;
}


.fa-home::before, .icon-home::before {
    content: "";
}

.wy-nav-top a {
    margin: -2em;
    background: transparent url(logo-small.svg) center left no-repeat;
    background-size: 1.5em;
    padding: 4px 4px 4px 26px;
}
